'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

const useIsMounted = () => {
    const isMounted = react.useRef(false);
    react.useEffect(() => {
        isMounted.current = true;
        return () => {
            isMounted.current = false;
        };
    }, []);
    return isMounted;
};
function lipidReactHookGenerator(lipidState) {
    return (properties) => {
        const isMounted = useIsMounted();
        const [state, setState] = react.useState(Object.fromEntries(properties.reduce((acc, key) => {
            acc.push([key, lipidState.get(key)]);
            return acc;
        }, [])));
        react.useEffect(() => {
            const obs = lipidState
                .on(properties)
                .subscribe(({ state }) => {
                if (isMounted.current) {
                    setState(state);
                }
            });
            return () => {
                obs.unsubscribe();
            };
        }, []);
        return state;
    };
}

exports.lipidReactHookGenerator = lipidReactHookGenerator;
